#: L23 Port Match Term Commands

from dataclasses import dataclass
import typing

from ..protocol.command_builders import (
    build_get_request,
    build_set_request
)
from .. import interfaces
from ..transporter.token import Token
from ..protocol.fields import data_types as xt
from ..protocol.fields.field import XmpField
from ..registry import register_command
from .enums import *  # noqa: F403


@register_command
@dataclass
class PM_INDICES:
    """
    The full list of which match terms are defined for a port. These are the sub-
    index values that are used for the parameters defining the content-based
    matching of packets received for the port. Setting the value of this parameter
    creates a new empty match term for each value that is not already in use, and
    deletes each match term that is not mentioned in the list. The same can be
    accomplished one match-term-at-a-time using the PM_CREATE and PM_DELETE commands.
    """

    code: typing.ClassVar[int] = 200
    pushed: typing.ClassVar[bool] = True

    _connection: "interfaces.IConnection"
    _module: int
    _port: int

    @dataclass(frozen=True)
    class SetDataAttr:
        match_term_xindices: XmpField[xt.XmpIntList] = XmpField(xt.XmpIntList)  # list of integers, the sub-index of a match term definition for the port.

    @dataclass(frozen=True)
    class GetDataAttr:
        match_term_xindices: XmpField[xt.XmpIntList] = XmpField(xt.XmpIntList)  # list of integers, the sub-index of a match term definition for the port.

    def get(self) -> "Token[GetDataAttr]":
        """Get the full list of which match terms are defined for a port.

        :return: the full list of which match terms are defined for a port
        :rtype: PM_INDICES.GetDataAttr
        """
        return Token(self._connection, build_get_request(self, module=self._module, port=self._port))

    def set(self, match_term_xindices: typing.List[int]) -> "Token":
        """Creates a new empty match term for each value that is not already in use, and delete each match term that is not mentioned in the list

        :param match_term_xindices: the sub-index of a match term definition for the port
        :type match_term_xindices: List[int]
        """
        return Token(self._connection, build_set_request(self, module=self._module, port=self._port, match_term_xindices=match_term_xindices))


@register_command
@dataclass
class PM_CREATE:
    """
    Creates an empty match term definition with the specified sub-index value.
    """

    code: typing.ClassVar[int] = 201
    pushed: typing.ClassVar[bool] = False

    _connection: "interfaces.IConnection"
    _module: int
    _port: int
    _match_term_xindex: int

    @dataclass(frozen=True)
    class SetDataAttr:
        pass

    def set(self) -> "Token":
        """Creates an empty match term definition with the specified sub-index value.
        """
        return Token(
            self._connection,
            build_set_request(
                self,
                module=self._module,
                port=self._port,
                indices=[self._match_term_xindex],
            ),
        )


@register_command
@dataclass
class PM_DELETE:
    """
    Deletes the match term definition with the specified sub-index value. A match
    term cannot be deleted while it is used in the condition of any filter for the
    port.
    """

    code: typing.ClassVar[int] = 202
    pushed: typing.ClassVar[bool] = False

    _connection: "interfaces.IConnection"
    _module: int
    _port: int
    _match_term_xindex: int

    @dataclass(frozen=True)
    class SetDataAttr:
        pass

    def set(self) -> "Token":
        """Deletes the match term definition with the specified sub-index value.
        """
        return Token(
            self._connection,
            build_set_request(
                self,
                module=self._module,
                port=self._port,
                indices=[self._match_term_xindex],
            ),
        )


@register_command
@dataclass
class PM_PROTOCOL:
    """
    The protocol segments assumed on the packets received on the port. This is
    mainly for information purposes, and helps you identify which portion of the
    packet header is being matched. The actual value definition of the match
    position is specified with PM_POSITION.
    """

    code: typing.ClassVar[int] = 203
    pushed: typing.ClassVar[bool] = True

    _connection: "interfaces.IConnection"
    _module: int
    _port: int
    _match_term_xindex: int

    @dataclass(frozen=True)
    class SetDataAttr:
        segments: XmpField[xt.XmpByteList] = XmpField(
            xt.XmpByteList, choices=ProtocolOption
        )  # list of coded bytes, a number specifying a built-in protocol segment: Uses the same coded values as the PS_HEADERPROTOCOL parameter.

    @dataclass(frozen=True)
    class GetDataAttr:
        segments: XmpField[xt.XmpByteList] = XmpField(
            xt.XmpByteList, choices=ProtocolOption
        )  # list of coded bytes, a number specifying a built-in protocol segment: Uses the same coded values as the PS_HEADERPROTOCOL parameter.

    def get(self) -> "Token[GetDataAttr]":
        """Get the protocol segments assumed on the packets received on the port.

        :return: a number specifying a built-in protocol segment: Uses the same coded values as the PS_HEADERPROTOCOL parameter.
        :rtype: PM_PROTOCOL.GetDataAttr
        """
        return Token(self._connection, build_get_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex]))

    def set(self, segments: typing.List[ProtocolOption]) -> "Token":
        """Set the protocol segments assumed on the packets received on the port.

        :param segments: a number specifying a built-in protocol segment: Uses the same coded values as the PS_HEADERPROTOCOL parameter
        :type segments: List[ProtocolOption]
        """
        return Token(self._connection, build_set_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex], segments=segments))


@register_command
@dataclass
class PM_POSITION:
    """
    The position within each received packet where content matching begins for the port.
    """

    code: typing.ClassVar[int] = 204
    pushed: typing.ClassVar[bool] = True

    _connection: "interfaces.IConnection"
    _module: int
    _port: int
    _match_term_xindex: int

    @dataclass(frozen=True)
    class SetDataAttr:
        byte_offset: XmpField[xt.XmpInt] = XmpField(xt.XmpInt)  # integer, offset from the start of the packet bytes.

    @dataclass(frozen=True)
    class GetDataAttr:
        byte_offset: XmpField[xt.XmpInt] = XmpField(xt.XmpInt)  # integer, offset from the start of the packet bytes.

    def get(self) -> "Token[GetDataAttr]":
        """Get the position within each received packet where content matching begins for the port.

        :return: offset from the start of the packet bytes
        :rtype: PM_POSITION.GetDataAttr
        """
        return Token(self._connection, build_get_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex]))

    def set(self, byte_offset: int) -> "Token":
        """Set the position within each received packet where content matching begins for the port.

        :param byte_offset: offset from the start of the packet bytes
        :type byte_offset: int
        """
        return Token(self._connection, build_set_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex], byte_offset=byte_offset))


@register_command
@dataclass
class PM_MATCH:
    """
    The value that must be found at the match term position for packets received on
    the port. The mask can make certain bit positions don't-care.
    """

    code: typing.ClassVar[int] = 205
    pushed: typing.ClassVar[bool] = True

    _connection: "interfaces.IConnection"
    _module: int
    _port: int
    _match_term_xindex: int

    @dataclass(frozen=True)
    class SetDataAttr:
        mask: XmpField[xt.XmpHex8] = XmpField(xt.XmpHex8)  # eight hex bytes, which bits are significant in the match operation.
        value: XmpField[xt.XmpHex8] = XmpField(xt.XmpHex8)  # eight hex bytes, the value that must be found for the match term to be true.

    @dataclass(frozen=True)
    class GetDataAttr:
        mask: XmpField[xt.XmpHex8] = XmpField(xt.XmpHex8)  # eight hex bytes, which bits are significant in the match operation.
        value: XmpField[xt.XmpHex8] = XmpField(xt.XmpHex8)  # eight hex bytes, the value that must be found for the match term to be true.

    def get(self) -> "Token[GetDataAttr]":
        """Get the value that must be found at the match term position for packets received on the port.

        :return: which bits are significant in the match operation, and the value that must be found for the match term to be true.
        :rtype: PM_MATCH.GetDataAttr
        """
        return Token(self._connection, build_get_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex]))

    def set(self, mask: str, value: str) -> "Token":
        """Set the value that must be found at the match term position for packets received on the port.

        :param mask: which bits are significant in the match operation
        :type mask: str
        :param value: the value that must be found for the match term to be true
        :type value: str
        """
        return Token(self._connection, build_set_request(self, module=self._module, port=self._port, indices=[self._match_term_xindex], mask=mask, value=value))



from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from xoa_driver.internals.core import interfaces as itf
from xoa_driver.internals.core.commands import (
    PP_LINKFLAP_PARAMS,
    PP_LINKFLAP_ENABLE,
    PP_PMAERRPUL_PARAMS,
    PP_PMAERRPUL_ENABLE,
)


class LinkFlap:
    """L23 high-speed port link flap."""

    def __init__(self, conn: "itf.IConnection", module_id: int, port_id: int) -> None:
        self.params = PP_LINKFLAP_PARAMS(conn, module_id, port_id)
        """Link flap parameters.
        Representation of PP_LINKFLAP_PARAMS
        """

        self.enable = PP_LINKFLAP_ENABLE(conn, module_id, port_id)
        """Link flap control.
        Representation of PP_LINKFLAP_ENABLE
        """


class PmaPulseErrInj:
    """L23 high-speed port PMA pulse error injection."""

    def __init__(self, conn: "itf.IConnection", module_id: int, port_id: int) -> None:
        self.params = PP_PMAERRPUL_PARAMS(conn, module_id, port_id)
        """PMA pulse error injection parameters.
        Representation of PP_PMAERRPUL_PARAMS
        """

        self.enable = PP_PMAERRPUL_ENABLE(conn, module_id, port_id)
        """PMA pulse error injection control.
        Representation of PP_PMAERRPUL_ENABLE
        """


class PcsPma:
    """PCS/PMA settings"""

    def __init__(self, conn: "itf.IConnection", port) -> None:
        self.link_flap = LinkFlap(conn, *port.kind)
        """Link flap settings."""

        self.pma_pulse_err_inj = PmaPulseErrInj(conn, *port.kind)
        """PMA pulse error injection settings."""



from __future__ import annotations
from typing import Type


class RegisterModule:
    """Register revision and it's module class"""

    def __init__(self, modules_store: dict[str, Type], rev: str) -> None:
        self.modules_store = modules_store
        self.revision = rev

    def __call__(self, module_type: Type):
        if self.revision in self.modules_store:
            raise RuntimeError(f"Module of revision: {self.revision}, is already registered.")
        self.modules_store[self.revision] = module_type
        return module_type


# register_valkyrie_module = partial(RegisterModule, modules_store=VALKYRIE_MODULES)
# register_chimera_module = partial(RegisterModule, modules_store=CHIMERA_MODULES)
# register_vulcan_module = partial(RegisterModule, modules_store=VULCAN_MODULES)

# # region Valkyrie Modules, sorting order: alphabetical

# VALKYRIE_MODULES = {
#     # family_d
#     "Odin-1G-3S-2P-T": modules.MOdin1G3S2PT,
#     "Odin-1G-3S-6P": modules.MOdin1G3S6P,
#     "Odin-1G-3S-6P[b]": modules.MOdin1G3S6P_b,
#     "Odin-1G-3S-6P-E": modules.MOdin1G3S6PE,

#     # family_m
#     "Odin-1G-3S-6P-T1-RJ45": modules.MOdin1G3S6PT1RJ45,

#     # family_e
#     "Odin-5G-4S-6P-CU": modules.MOdin5G4S6PCU,

#     # family_f
#     "Odin-10G-1S-2P": modules.MOdin10G1S2P,
#     "Odin-10G-1S-2P[b]": modules.MOdin10G1S2P_b,
#     "Odin-10G-1S-2P[c]": modules.MOdin10G1S2P_c,
#     "Odin-10G-1S-2P[d]": modules.MOdin10G1S2P_d,
#     "Odin-10G-1S-2P-T": modules.MOdin10G1S2PT,
#     "Odin-10G-1S-6P": modules.MOdin10G1S6P,
#     "Odin-10G-1S-6P[b]": modules.MOdin10G1S6P_b,
#     "Odin-10G-1S-12P": modules.MOdin10G1S12P,

#     # family_e
#     "Odin-10G-3S-2P-CU": modules.MOdin10G3S2PCU,
#     "Odin-10G-3S-6P-CU": modules.MOdin10G3S6PCU,

#     "Odin-10G-5S-6P-CU": modules.MOdin10G5S6PCU,
#     "Odin-10G-5S-6P-CU[b]": modules.MOdin10G5S6PCU_b,

#     # family_combi
#     "Odin-10G-4S-2P-Combi": modules.MOdin10G4S2PCombi,
#     "Odin-10G-4S-2P-Combi[b]": modules.MOdin10G4S2PCombi_b,

#     # family_n
#     "Odin-40G-2S-2P": modules.MOdin40G2S2P,
#     "Odin-40G-2S-2P-B": modules.MOdin40G2S2PB,

#     # family_h
#     "Odin-100G-3S-1P": modules.MThor400G7S1P_d,

#     # family_g
#     "Loki-100G-3S-1P": modules.MLoki100G3S1P,
#     "Loki-100G-3S-1P[b]": modules.MLoki100G3S1P_b,
#     "Loki-100G-3S-1P-SE": modules.MLoki100G3S1PSE,
#     "Loki-100G-3S-1P-B": modules.MLoki100G3S1PB,

#     # family_h
#     "Loki-100G-5S-1P": modules.MLoki100G5S1P,

#     # family_i
#     "Loki-100G-5S-2P": modules.MLoki100G5S2P,

#     # family_j
#     "Thor-100G-5S-4P": modules.MThor100G5S4P,

#     # family_k
#     "Thor-400G-7S-1P": modules.MThor400G7S1P,

#     # family_l
#     "Thor-400G-7S-1P[b]": modules.MThor400G7S1P_b,
#     "Thor-400G-7S-1P[c]": modules.MThor400G7S1P_c,
#     "Thor-400G-7S-1P[d]": modules.MThor400G7S1P_d,

#     # family_l1
#     "Freya-800G-1S-1P[a]": modules.MFreya800G1S1P_a,
#     "Freya-800G-1S-1P[b]": modules.MFreya800G1S1P_b,
#     "Freya-800G-1S-1P-OSFP[a]": modules.MFreya800G1S1POSFP_a,
#     "Freya-800G-4S-1P[a]": modules.MFreya800G4S1P_a,
#     "Freya-800G-4S-1P[b]": modules.MFreya800G4S1P_b,
#     "Freya-800G-4S-1P-OSFP[a]": modules.MFreya800G4S1POSFP_a,
# }

# # endregion

# # region Chimera Modules

# CHIMERA_MODULES = {
#     "Chimera-100G-5S-2P": modules.MChi100G5S2P,
#     "Chimera-100G-5S-2P[b]": modules.MChi100G5S2P_b,
#     "Chimera-40G-2S-2P": modules.MChi40G2S2P,
# }

# # endregion

# # region Vulcan Modules

# VULCAN_MODULES = {
#     "Xena L47 Module": modules.ModuleL47,
# }

# # endregion
